/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER             "0.1"
#define BUFFSZ          4096
#define POPUSER         "POPUser="
#define SMTPUSER        "SMTPUser="
#define POPPASS         "POPPass="
#define SMTPPASS        "SMTPPass="
#define POPSERVER       "POPServer="
#define SMTP            "SMTP="
#define ENCPASS         "EncryptPass="
#define CHECK(x)        if(!strnicmp(buff, x, sizeof(x) - 1))



#ifndef WIN32
    #define strnicmp strncasecmp
#endif

#ifndef u_char
    #define u_char  unsigned char
#endif



void poco_pwd(u_char *pwd, int type);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  *buff,
            *ptr;


    setbuf(stdout, NULL);

    fputs("\n"
        "Pocomail passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <accounts.ini>\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("\nOpening file %s\n\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    buff = malloc(BUFFSZ + 1);
    if(!buff) std_err();

    while(1) {
        if(!fgets(buff, BUFFSZ, fd)) break;

        if(*buff == '[') {
            printf("\n%s", buff);
        } else CHECK(POPUSER) {
            fputs(buff, stdout);
        } else CHECK(SMTPUSER) {
            fputs(buff, stdout);
        } else CHECK(POPPASS) {
            for(ptr = buff; *ptr > 0xd; ptr++);
            *ptr = 0;
            poco_pwd(buff + sizeof(POPPASS) - 1, 0);
            printf("%s\n", buff);
        } else CHECK(SMTPPASS) {
            for(ptr = buff; *ptr > 0xd; ptr++);
            *ptr = 0;
            poco_pwd(buff + sizeof(SMTPPASS) - 1, 0);
            printf("%s\n", buff);
        } else CHECK(POPSERVER) {
            fputs(buff, stdout);
        } else CHECK(SMTP) {
            fputs(buff, stdout);
        } else CHECK(ENCPASS) {
            for(ptr = buff; *ptr > 0xd; ptr++);
            *ptr = 0;
            poco_pwd(buff + sizeof(ENCPASS) - 1, 1);
            printf("%s\n", buff);
        }
    }

    fclose(fd);
    fputs("\n\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void poco_pwd(u_char *pwd, int type) {
    int     len,
            tmp;
    u_char  *out;
    short   azz;

    if(type) azz = 0x2537;  // encrypt message
        else azz = 0x2a9a;  // other passwords

    len = strlen(pwd) >> 1;
    out = pwd;
    while(len--) {
        sscanf(pwd, "%02X", &tmp);
        pwd += 2;
        *out++ = tmp ^ (azz >> 8);
        azz = ((tmp + azz) * 0x8141) + 0x3171;
    }
    *out = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


